@@ -69,8 +69,8 @@ count_to_n(bitarrayobject *a, Py_ssize_t n)
     while (i + BLOCK_BITS < a->nbits) {
         m = 0;
         assert(i % 8 == 0);
-        block_start = i / 8;
-        block_stop = block_start + (BLOCK_BITS / 8);
+        block_start = i >> 3;
+        block_stop = block_start + (BLOCK_BITS >> 3);
         assert(block_stop <= Py_SIZE(a));
         for (k = block_start; k < block_stop; k++)
             m += bitcount_lookup[(unsigned char) a->ob_item[k]];
@@ -82,7 +82,7 @@ count_to_n(bitarrayobject *a, Py_ssize_t n)
 #undef BLOCK_BITS
 
     while (i + 8 < a->nbits) {
-        k = i / 8;
+        k = i >> 3;
         assert(k < Py_SIZE(a));
         m = bitcount_lookup[(unsigned char) a->ob_item[k]];
         if (j + m >= n)
@@ -149,32 +149,31 @@ find_last(bitarrayobject *a, int vi)
     if (a->nbits == 0)
         return -1;
 
-    /* search within top byte */
-    for (i = a->nbits - 1; i >= BITS(a->nbits / 8); i--)
+    /* search within highest (partial) byte */
+    for (i = a->nbits - 1; i >= BITS(a->nbits >> 3); i--)
         if (getbit(a, i) == vi)
             return i;
-
-    if (i < 0)  /* not found within top byte */
+    if (i < 0)  /* not found */
         return -1;
-    assert((i + 1) % 8 == 0);
+    assert((i + 1) % 8 == 0 && i == BITS(a->nbits >> 3) - 1);
 
     /* seraching for 1 means: break when byte is not 0x00
        searching for 0 means: break when byte is not 0xff */
     c = vi ? 0x00 : 0xff;
 
     /* skip ahead by checking whole bytes */
-    for (j = BYTES(i) - 1; j >= 0; j--)
+    for (j = i >> 3; j >= 0; j--) {
+        assert(0 <= j && j < Py_SIZE(a));
         if (c ^ a->ob_item[j])
             break;
-
-    if (j < 0)  /* not found within bytes */
-        return -1;
-
+    }
     /* search within byte found */
-    for (i = BITS(j + 1) - 1; i >= BITS(j); i--)
-        if (getbit(a, i) == vi)
+    for (i = BITS(j) + 7; i >= 0; i--)
+        if (getbit(a, i) == vi) {
+            assert(BITS(j) <= i && i < BITS(j + 1));
             return i;
-
+        }
+    assert(j < 0 && i < 0);
     return -1;
 }
 
@@ -620,7 +619,11 @@ base2ba(PyObject *module, PyObject *args)
 
 /* PADBITS is always 3 - the number of bits that represent the number of
    padding bits.  The actual number of padding bits is called 'padding'
-   below, and is in range(0, 7). */
+   below, and is in range(0, 7).
+   Also note that 'padding' refers to the pad bits within the variable
+   length format, which is not the same as the pad bits of the actual
+   bitarray.  For example, b'\x10' has padding = 1, and decodes to
+   bitarray('000'), which has 5 pad bits. */
 #define PADBITS  3
 
 /* Consume iterator while decoding bytes into bitarray.
@@ -738,19 +741,20 @@ vl_encode(PyObject *module, PyObject *a)
         return PyErr_NoMemory();
 
     str[0] = aa->nbits > 4 ? 0x80 : 0x00;  /* leading bit */
-    str[0] |= padding << 4;                /* encode number of pad bits */
+    str[0] |= padding << 4;                /* encode padding */
     for (i = 0; i < 4 && i < aa->nbits; i++)
         str[0] |= (0x08 >> i) * getbit(aa, i);
 
     for (i = 4; i < aa->nbits; i++) {
         k = (i - 4) % 7;
-        if (k == 0)
-            str[++j] = i + 7 < m ? 0x80 : 0x00;  /* leading bit */
-
+        if (k == 0) {
+            j++;
+            str[j] = j < n - 1 ? 0x80 : 0x00;  /* leading bit */
+        }
         str[j] |= (0x40 >> k) * getbit(aa, i);
     }
 #undef aa
-    assert(j + 1 == n);
+    assert(j == n - 1);
 
     result = PyBytes_FromStringAndSize(str, n);
     PyMem_Free((void *) str);
