@@ -4,7 +4,7 @@
 
    Author: Ilan Schnell
 */
-#define BITARRAY_VERSION  "2.2.3"
+#define BITARRAY_VERSION  "2.2.4"
 
 /* .ob_size is buffer size (in bytes), not the number of elements.
    The number of elements (bits) is .nbits. */
@@ -37,9 +37,9 @@ typedef struct {
 /* assert that .nbits is in agreement with .ob_size */
 #define assert_nbits(self)  assert(BYTES((self)->nbits) == Py_SIZE(self))
 
-/* assert index i is in range */
-#define assert_bit_in_range(self, i)  assert(0 <= (i) && (i) < (self)->nbits)
-#define assert_byte_in_range(self, i)  assert(0 <= (i) && (i) < Py_SIZE(self))
+/* assert byte index is in range */
+#define assert_byte_in_range(self, j)  \
+    assert(self->ob_item && 0 <= (j) && (j) < Py_SIZE(self))
 
 /* --------------- definitions not specific to bitarray ---------------- */
 
@@ -65,8 +65,9 @@ typedef struct {
 #define IS_PY3K
 #define BYTES_SIZE_FMT  "y#"
 #else
-/* the Py_MIN macro was introduced in Python 3.3 */
+/* the Py_MIN and Py_MAX macros were introduced in Python 3.3 */
 #define Py_MIN(x, y)  (((x) > (y)) ? (y) : (x))
+#define Py_MAX(x, y)  (((x) > (y)) ? (x) : (y))
 #define PySlice_GetIndicesEx(slice, len, start, stop, step, slicelength) \
     PySlice_GetIndicesEx(((PySliceObject *) slice),                      \
                          (len), (start), (stop), (step), (slicelength))
@@ -80,8 +81,8 @@ static inline int
 getbit(bitarrayobject *self, Py_ssize_t i)
 {
     assert_nbits(self);
-    assert_bit_in_range(self, i);
     assert_byte_in_range(self, i >> 3);
+    assert(0 <= i && i < self->nbits);
     return (self->ob_item[i >> 3] & BITMASK(self->endian, i) ? 1 : 0);
 }
 
@@ -91,8 +92,8 @@ setbit(bitarrayobject *self, Py_ssize_t i, int bit)
     char *cp, mask;
 
     assert_nbits(self);
-    assert_bit_in_range(self, i);
     assert_byte_in_range(self, i >> 3);
+    assert(0 <= i && i < self->nbits);
     mask = BITMASK(self->endian, i);
     cp = self->ob_item + (i >> 3);
     if (bit)
@@ -116,7 +117,7 @@ setunused(bitarrayobject *self)
         return 0;
 
     assert_nbits(self);
-    assert_byte_in_range(self, Py_SIZE(self) - 1);
+    assert(self->ob_item && Py_SIZE(self) > 0);
     /* apply the appropriate mask to the last byte in buffer */
     self->ob_item[Py_SIZE(self) - 1] &=
         mask[self->endian == ENDIAN_LITTLE ? i - 1 : i + 6];
